function bintp(xx,n,x,y)
! 2nd order polynomial interpolation
!--- history
! 88. 1.14 created
!     3. 7 linear extrapolation for out-ou-boundary data
! 08. 9.19 modified Fortran 90 free-style form
!--- input
! xx    r     interpolation point
! n     r     nbr of data
! x   r(nn)   independent variable,  x(i) .lt. x(i+1)
! y   r(nn)   dependent   variable
!--- output
! bintp r     output
!--
  implicit none

! for input & output
  integer,intent(in)::n
  real(8),intent(in)::xx,x(n),y(n)
  real(8):: bintp
! for work
  integer::i,i1,i2,i3,k1,k2
!--exec
! 1 point
  if(n <= 1) then
     bintp=y(1)
     return
  endif

! 2 points or xx.le.x(1)
  if(xx <= x(1) .or. n==2) then
     bintp=y(1)+(y(2)-y(1))*(xx-x(1))/(x(2)-x(1))
     return
  endif
! xx.ge.xx(n)
  if(xx > x(n)) then
     bintp=y(n-1)+(y(n)-y(n-1))*(xx-x(n-1))/(x(n)-x(n-1))
     return
  endif

! 3 points
  do i=1,n
     if(xx < x(i)) goto 10
  enddo
  i=n
10 k1=max(1,i-1); k2=k1+2
  if(k2 > n) k2=n; k1=k2-2

  bintp=0.0d0; i2=k1; i3=k1+1
  do i=1,3
     i1=i2; i2=i3; i3=mod(i+1,3)+k1
     bintp=bintp+(xx-x(i2))*(xx-x(i3))/(x(i1)-x(i2))/(x(i1)-x(i3))*y(i1)
  enddo
  return
end function bintp
subroutine cspl1(n,x,y,a,b,c,d)
! getting coefficients of natural cubic spline fitting
! use a linear interpolation outside the meaningful range of x
! x-increasing order
!--- history
! 88. 1. 4  created
! 08.10.29 modified fortran 90 version.
!--- input
! n      i     nbr of data
! x    r(n)    independent variable data
!              x-increasing order
! y    r(n)      dependent variable data
!--- output
! a    r(n)    lambda -> a   where  y=a+x*(b+x*(c+d*x))
! b    r(n)    d      -> b   i-th for range (x(i-1), x(i))
! c    r(n)    m         c   (a,b,c,d for i=1 are same as those for i=2)
! d    r(n)    m         d
!--- notes
! ref-1   p. f. davis and philip rabinowitz (1984)
!         methods pf numerical integration, second edition
!         academic press, inc., pp612.
!--
  implicit none
  integer,intent(in)::n
  real(8),intent(in)::x(n),y(n)
  real(8),intent(out)::a(n),b(n),c(n),d(n)
  integer::i
  real(8)::h1,h2,s1,s2,q1,q2,u1,u2,p1,p2,c1,c2,x1,x2,x3,x4,z1,z2,z3,z4
!--
  if(n > 4) then
! n > 4
     a(1)=0.d0; a(n)=1.d0; b(1)=0.d0; b(n)=0.d0
     h2=x(2)-x(1); s2=(y(2)-y(1))/h2
     do i=2,n-1
        h1=h2; h2=x(i+1)-x(i)
        s1=s2; s2=(y(i+1)-y(i))/h2
        a(i)=1.d0/(1.d0+h1/h2)
        b(i)=6.d0*(s2-s1)/(h2+h1)
     enddo

     q2=0.d0; u2=0.d0
     do i=1,n
        q1=q2; u1=u2
        p2=(1.d0-a(i))*q1+2.d0
        q2=-a(i)/p2; u2=(b(i)-(1.0-a(i))*u1)/p2
        a(i)=q2; b(i)=u2
     enddo
     c(n)=b(n)
     do i=n-1,1,-1
        c(i)=a(i)*c(i+1)+b(i)
     enddo

     x2=x(1); c2=c(1)/6.d0
     do i=2,n
        h2=x(i)-x(i-1); x1=x2
        x2=x(i); c1=c2; c2=c(i)/6.d0
        p1=y(i-1)/h2-c1*h2; p2=y(i  )/h2-c2*h2
        a(i)=     ( c1*x2**3-c2*x1**3)/h2+p1*x2-p2*x1
        b(i)=3.d0*(-c1*x2**2+c2*x1**2)/h2-p1   +p2
        c(i)=3.d0*( c1*x2   -c2*x1   )/h2
        d(i)=     (-c1      +c2      )/h2
     enddo

  elseif(n==1) then
! n=1
     a(1)=y(1); b(1)=0.d0; c(1)=0.d0; d(1)=0.d0
     return

  elseif(n==2) then
! n=2
     x1=x(1); x2=x(2); z1=y(1)/(x1-x2); z2=y(2)/(x2-x1)
     a(2)=-x2*z1-x1*z2; b(2)=z1+z2; c(2)=0.d0; d(2)=0.d0

  elseif(n==3) then
! n=3
     x1=x(1); x2=x(2); x3=x(3)
     z1=y(1)/(x1-x2)/(x1-x3); z2=y(2)/(x2-x3)/(x2-x1)
     z3=y(3)/(x3-x1)/(x3-x2)
     a(2)=x2*x3*z1+x3*x1*z2+x1*x2*z3
     b(2)=-(x2+x3)*z1-(x3+x1)*z2-(x1+x2)*z3
     c(2)=z1+z2+z3; d(2)=0.d0
     a(3)=a(2); b(3)=b(2); c(3)=c(2); d(3)=d(2)

  elseif(n==4) then
! n=4
     x1=x(1); x2=x(2); x3=x(3); x4=x(4)
     z1=y(1)/(x1-x2)/(x1-x3)/(x1-x4)
     z2=y(2)/(x2-x3)/(x2-x4)/(x2-x1)
     z3=y(3)/(x3-x4)/(x3-x1)/(x3-x2)
     z4=y(4)/(x4-x1)/(x4-x2)/(x4-x3)
     a(2:4)=-x2*x3*x4*z1-x3*x4*x1*z2-x4*x1*x2*z3-x1*x2*x3*z4
     b(2:4)=(x2*x3+x3*x4+x4*x2)*z1+(x3*x4+x4*x1+x1*x3)*z2 &
          +(x4*x1+x1*x2+x2*x4)*z3+(x1*x2+x2*x3+x3*x1)*z4
     c(2:4)=-(x2+x3+x4)*z1-(x3+x4+x1)*z2-(x4+x1+x2)*z3-(x1+x2+x3)*z4
     d(2:4)=z1+z2+z3+z4
  endif

  a(1)=a(2); b(1)=b(2); c(1)=c(2); d(1)=d(2)
  return
end subroutine cspl1
function cspli(x1,n,x,a,b,c,d)
! get interpolated value using cubic spline (pair with cspl1)
!--- history
! 88. 6. 6  registered by t. nakajima
! 08.10.29 modified fortran 90 version.
!--- input
! x1      r     interpolation point
! n       i     no. of intervals + 1
! x     r(n)    division points of the intervals.
! a, b, c, d
!       r(n)    y=a+x*(b+x*(c+x*d)))
!--- output
! cspli  rf     interpolated value
!--
  implicit none
  integer,intent(in)::n
  real(8),intent(in)::x1,x(n),a(n),b(n),c(n),d(n)
  integer::j
  real(8)::cspli
!--exec
  do j=1,n
     if(x1 <= x(j)) then
        cspli=a(j)+x1*(b(j)+x1*(c(j)+x1*d(j)))
        return
     endif
  enddo
  cspli=a(n)+x1*(b(n)+x1*(c(n)+x1*d(n)))
  return
end function cspli
function erfc(x)
! complementary error function=(2/sqrt(pi) integral(x,inf) exp(-t**2)dt
! 92.12.22: created by Nakajima
!--- input
! x       r      independent variable 0 to inf
!--- output
! erfc   rf      complementary error function
!--
  implicit none

! input & output
  real(8),intent(in)::x
  real(8):: erfc

! work
  real(8),parameter:: c1 = 7.05230784d-2
  real(8),parameter:: c2 = 4.22820123d-2
  real(8),parameter:: c3 = 9.2705272d-3
  real(8),parameter:: c4 = 1.520143d-4
  real(8),parameter:: c5 = 2.765672d-4
  real(8),parameter:: c6 = 4.30638d-5

  real(8)::v
!--
  v=abs(x)
  if(v <= 7.d0) then
     erfc=1.d0/(1.d0+v*(c1+v*(c2+v*(c3+v*(c4+v*(c5+c6*v))))))**16
  else
     erfc=0.d0
  endif
  if(x < 0.d0) erfc=2.d0-erfc
  return
end function erfc
function expfn(x)
! exponential function with over/under flow setting.
!--- history
! 89. 5. 4   created by T. Nakajima.
! 90. 1.17   updated with cpcon
! 03. 9. 9   changed fpr free source term
!--- input
! x        R         independent variable.
!--- output
! expfn    F         exp(x).
!                     if x.le. vmn then exp(x) is reset as   0.
!                     if x.ge. vmx then exp(x) is reset as exp(vmx).
!--- parameters
! system set the -vmn- and -vmx- by the function r1mach.
!--
  implicit none
  integer,save::init
  real(8),save::vmn,vmx,expmn,expmx
  real(8),intent(in)::x
  real(8)::ccp(3),expfn
!--exec
  init=1

! set vmn      r         enderflow limit.
!     vmx      r         overflow limt.
  if(init > 0) then
     init=0
     call cpcon(ccp)
     vmn=ccp(2)*0.8*2.3; vmx=ccp(3)*0.8*2.3
     expmn=0.0d0; expmx=exp(vmx)
  endif

  if(x <= vmn) then
     expfn=expmn
  else
     if(x >= vmx) then
        expfn=expmx
     else
        expfn=exp(x)
     endif
  endif
  return
end function expfn


subroutine cpcon(c)
! machine constants of computer
!--- output
! c     r(3)      (1)  minimum positive x for  1+x      .ne. 1
!                      average as a result of complex arithmetic
!                      operations.
!                 (2)  minimum exponent y for  10.0**y  .ne. 0
!                 (3)  maximum exponent z for  10.0**z  is max. value
!                  if init=1 (data statement) then  set as z=y
!                  if init=2 then this routine gets actual value of z.
!                  - see note -
!--- history
! 90. 1.20  created
!     6.27  change the algorithm to get x and y taking into account the
!           high accuracy co-processor and graceful underflow.
! 92. 3.21  n=1000 from n=200
!     7. 9  bug in x-definition
! 03. 9. 9   changed fpr free source term
!--- note
! this program will generate -underflow error- and -overflow error-
!  messages.  on some computer -overflow error- message may be
!  fatal error.  in that case, please set init = 1 in the data
!  satement for suppressing the procedure of getting c(3).
!--
  use VIRparameters, only: pi
  implicit none
  real(8),intent(out)::c(3)
  real(8),save::x,y,z
  integer,save::init=1
  real(8)::y1,y2,y3,z1,z2
  integer::n,m0,m,k,i
  character ch*80
!--exec
! resolution of computation
  if(init <= 0) then
     c(1)=x; c(2)=y; c(3)=z
     return
  endif

!! test sum(k=1,m) cos((2k-1)*pi/(2m+1)) = 0.5
!! simple check, x = x + e, is not valide when the computer
!!  use a high accurate co-processor.
  n=500; m0=10; x=0.d0
  do m=1,m0
     y=0.d0
     do k=1,m
        y=y+cos((2.d0*k-1)*pi/(2.d0*m+1))
     enddo
     y=abs(2.d0*y-1)
     x=x+y
  enddo
  x=x/m0
  c(1)=x

! exponent for minimum positive value
! this procedure will generate -underflow error message-
  y2=1.d0; n=1000
  do i=1,n
     y1=y2; y3=y1/10.d0
!! for graceful underflow
!! even y2 becomes 0 as output, y2 is not 0 inside
!! computer when graceful underflow is applied.
!! so we replace the value of y2 by output.
!         ch='0'
!         write(ch,107) y3
!  107    format(1p,e12.5)
!         y2=0
!         read(ch,*) y2
     y2=y3
     if(abs(10.d0*y2/y1-1.d0) > 5.d0*x) goto 104
  enddo
  i=n+1
104 y=1-i
  c(2)=y

! exponent for maximum positive value
! this procedure will generate -overflow message-
  if(init <= 1) then
     z=-y
  else
     z2=1
     do i=1,n
        z1=z2; z2=z1*10.d0
        if(abs(z2/z1/10.d0-1.d0) > 5.d0*x) goto 106
     enddo
     i=n+1
106  z=i-1
  endif
  c(3)=z

  init=0
  return
end subroutine cpcon
function plgd(init,x)
! Legendre polynomials
!--- history
! 87.11.12 created
! 90. 1.16 save statement
!--- input
! init   i    if 1 then l=0
!             if 0 then l=l+1
! x      r    (-1,1)
!--- out
! init=0
  implicit none
  integer,intent(inout)::init
  real(8),intent(in)::x
  integer,save::l
  real(8),save::pl,pl1,pl2
  real(8)::plgd
!--
  if(init > 0) then
     init=0; l=-1
  endif
  l=l+1
  if(l == 0) then
     pl=1
  elseif(l == 1) then
     pl1=pl; pl=x
  else
     pl2=pl1; pl1=pl
     pl=(dble(2*l-1)*x*pl1-dble(l-1)*pl2)/dble(l)
  endif
  plgd=pl
  return
end function plgd
subroutine plgnd(m1,mmx1,nx,nx0,x,pl)
! normalized associated legendre polynomials
!------ history
! 1987.03.04
! 08.10.24 modified fortran 90 version.
!------ input variables
! variable  type    interpretation
! m1         i      order of the fourier series + 1
! mmx1       i      max order of m  + 1
! nx         i      nbr of x
! nx0        i      declared nbr of nx
! x       r(nx)     xi, i=1,nx
!------ output variables
! pl      r(nx0,    normalized associated legendre polynomials
!           mmx1)
  use VIRparameters, only:pi
  implicit none
  integer,intent(in)::m1,mmx1,nx,nx0
  real(8),intent(in)::x(nx)
  real(8),intent(out)::pl(nx0,mmx1)
  real(8)::eta,epsi,expcc,c0,c1,c2
  integer::m,l1,l,k,j,i
!--exec
! normalized associated legendre polynomials.
!  cc=p(m,l,x)*sqrt((l-m)|/(l+m)|), where x= cos(mu).
  m=m1-1
  do l1=m1,mmx1
     l=l1-1
     if(l < (m+1)) then
        if(m <= 0) then
           pl(1:nx,l1)=1.d0
        else
           k=2; eta=1.d0
           do j=1,m
              epsi=1.d0-1.d0/dble(k)
              eta=eta*epsi
              k=k+2
           enddo
           eta=sqrt(eta)
           do i=1,nx
              if(x(i) > 1.d0) then
                 pl(i,l1)=0.d0
              else
! / modified for SDSUv2.1.0 /
                 expcc=(dble(m)*log(max(abs(1.d0-x(i)**2),1.e-20)))/2.d0
!                 expcc=(dble(m)*log(abs(1.d0-x(i)**2)))/2.d0
                 if(expcc <= -100.d0) then
                    pl(i,l1)=0.d0
                 else
                    pl(i,l1)=eta*exp(expcc)
                 endif
              endif
           enddo
        endif
     elseif(l == m+1) then
        c0=sqrt(dble(2*m+1))
        pl(1:nx,l1)=c0*pl(1:nx,l1-1)*x(1:nx)
        c0=sqrt(dble((l-m)*(l+m)))
     else
        c1=sqrt(dble((l-m)*(l+m)))
        c2=dble(2*l-1)
        pl(1:nx,l1)=(c2*x(1:nx)*pl(1:nx,l1-1)-c0*pl(1:nx,l1-2))/c1
        c0=c1
     endif
  enddo
  return
end subroutine plgnd
subroutine  qgausn( gwt, gmu, m )
!  compute weights and abscissae for ordinary gaussian quadrature
!   (no weight function inside integral) on the interval (0,1)
!--- history
! 90. 1.17  registered
! 08. 9.25 modified Fortran 90 free-style form
!--- input
! m        i       order of quadrature rule
!--- output
! gmu    r(m)      array of abscissae (0, 1)
! gwt    r(m)      array of weights   sum=1
!--- notes
! reference:  Davis,p.j. and p. Rabinowitz, methods of numerical
!             integration,academic press, new york, 1975, pp. 87
! method:     compute the abscissae as roots of the legendre
!             polynomial p-sub-n using a cubically convergent
!             refinement of newton's method.  compute the
!             weights from eq. 2.7.3.8 of davis/rabinowitz.
!             accuracy:  at least 13 significant digits.
!--- internal variables
! pm2,pm1,p : 3 successive legendre polynomials
! ppr       : derivative of legendre polynomial
! p2pri     : 2nd derivative of legendre polynomial
! tol       : convergence criterion
! x,xi      : successive iterates in cubically-
!             convergent version of newton's method
!            ( seeking roots of legendre polynomial )
!--
  use VIRparameters, only:pi
  implicit none

! input & output
  integer,intent(inout)::m
  real(8),intent(out)::gmu(m),gwt(m)

! work
  integer::lim,np1,k,nn
  real(8)::cona, t, en, nnp1, p, pm1, pm2, ppr, p2pri, prod, tmp, x, xi
  real(8)::tol=1.0d-13
!--exec
  if ( m <= 1 )  then
     m = 1
     gmu( 1 ) = 0.5d0
     gwt( 1 ) = 1.0d0
     return
  end if

  en   = m
  np1  = m + 1
  nnp1 = m * np1
  cona = dble( m-1 ) / ( 8.d0 * m**3 )
!+---------------------------------------------------------------------+
!!         initial guess for k-th root of legendre polynomial,         |
!!         from davis/rabinowitz  eq. (2.7.3.3a)                       |
!+---------------------------------------------------------------------+
  lim  = m / 2
  do k = 1, lim
     t = dble( 4*k - 1 ) * pi / dble( 4*m + 2 )
     x = cos ( t + cona / tan( t ) )

!+---------------------------------------------------------------------+
!!             recursion relation for legendre polynomials             |
!!       initialize legendre polynomials: (pm2) p-sub-0, (pm1) p-sub-1 |
!+---------------------------------------------------------------------+
10   pm2 = 1.d0
     pm1 = x
     do nn = 2, m
!            p   = ( ( 2*nn - 1 ) * x * pm1 - ( nn-1 ) * pm2 ) / nn
        p   =  2* x * pm1 - pm2 - ( x * pm1 - pm2 ) / nn
        pm2 = pm1
        pm1 = p
     enddo

     tmp   = 1.d0 / ( 1.d0 - x**2 )
     ppr   = en * ( pm2 - x * p ) * tmp
     p2pri = ( 2.d0 * x * ppr - nnp1 * p ) * tmp
     xi    = x - ( p / ppr ) * ( 1.d0 + ( p / ppr ) &
          * p2pri / ( 2.d0 * ppr ) )

     if ( dabs(xi-x) > tol ) then
!!          check for convergence
        x = xi
        go to 10
     end if

!       ** iteration finished--calc. weights, abscissae for (-1,1)
     gmu( k ) = - x
     gwt( k ) = 2.d0 / ( tmp * ( en * pm2 )**2 )
     gmu( np1 - k ) = - gmu( k )
     gwt( np1 - k ) =   gwt( k )
  enddo

  if ( mod( m,2 ) /= 0 )  then
!!       set middle abscissa and weight for rules of odd order
     gmu( lim + 1 ) = 0.d0
     prod = 1.d0
     do k = 3, m, 2
        prod = prod * k / ( k-1 )
     enddo
     gwt( lim + 1 ) = 2.d0 / prod**2
  end if

  do k = 1, m
!!       convert from (-1,1) to (0,1)
     gmu( k ) = 0.5d0 * gmu( k ) + 0.5d0
     gwt( k ) = 0.5d0 * gwt( k )
  enddo
  return
end subroutine qgausn
subroutine symtrx(rowij,m,root,eigv,ni,wk,ier)
! solves eigenfunction problem for symmetric matrix
!--- history
! 89.12. 4 modified with cncpu
! 90. 1. 6 cncpu is replaced by prec.
! 08.10.29 modified fortran 90 version.
!--- input
! m       i      order of original symmetric matrix
! ni      i      initial dimension of -root-, -eigv- and -wk-
! rowij  r(*)    symmetric storage mode of order m*(m+1)/2
!--- output
! eigv   r(ni,m) eigenvectors of original symmetric matrix
! ier     i      index for root(j) failed to converge (j=ier-128)
! root   r(m)    eigenvalues of original symmetric matrix
! rowij          storage of householder reduction elements
! wk             work area
!--      
  use VIRparameters,only: krowij
  implicit none

! input & output
  integer,intent(in):: m
  integer,intent(in):: ni
  real(8),intent(inout):: rowij(krowij)
  real(8),intent(out):: root(ni)
  real(8),intent(out):: eigv(ni,ni)
  real(8),intent(out):: wk(ni)
  integer,intent(out):: ier

! work
  real(8)::ccp(3),rdelp,h,scale,f,g,hh,b,sum1,c,s,p,r
  integer::mp1,mm,mbeg,i,l,ii,mk,k,jk1,ik,jk,jp1,ia,nn1,j,n,l1,nn1pl
!--exec
!+++ add epscp
  call cpcon(ccp)
  rdelp=ccp(1)*10.d0
!+++
  ier = 0; mp1 = m + 1
  mm = (m*mp1)/2 - 1
  mbeg = mm + 1- m

!+---------------------------------------------------------------------+
!|          loop-100 reduce -rowij- (symmetric storage mode) to a      |
!|          symmetric tridiagonal form by householder method           |
!|                      cf. wilkinson, j.h., 1968,                     |
!|              the algebraic eigenvalue problem, pp 290-293.          |
!|          loop-30&40 and 50 form element of a*u and element p        |
!+---------------------------------------------------------------------+
! (loop-100)
  do ii=1,m
     i = mp1 - ii
     l = i - 1
     h = 0.d0; scale = 0.d0
     mk = mm
     do k=1,l
        scale = scale + abs(rowij(mk))
        mk = mk - 1
     enddo

     if (l < 1 .or. scale <= 0.0) then
! in l case   scale row (algol tol then not needed)
        wk(i) = 0.0
     else
        mk = mm
        do k = 1,l
           rowij(mk) = rowij(mk)/scale
           h = h + rowij(mk)*rowij(mk)
           mk = mk - 1
        enddo
        wk(i) = scale*scale*h
        f = rowij(mm)
        g = - sign(sqrt(h),f)
        wk(i) = scale*g
        h = h - f*g
        rowij(mm) = f - g
        if (l > 1) then
           f = 0.d0; jk1 = 1
! (loop-50)
           do j=1,l
              g = 0.d0
              ik = mbeg + 1; jk = jk1
! (loop-30)               
              do k=1,j
                 g = g + rowij(jk)*rowij(ik)
                 jk = jk + 1; ik = ik + 1
              enddo
              jp1 = j + 1
              if (l >= jp1) then
                 jk = jk + j - 1
! (loop-40)
                 do k=jp1,l
                    g = g + rowij(jk)*rowij(ik)
                    jk = jk + k; ik = ik + 1
                 enddo
              endif
              wk(j) = g/h
              f = f + wk(j)*rowij(mbeg+j)
              jk1 = jk1 + j
           enddo
           hh = f/(h+h)

           jk = 1
           do j=1,l
              f = rowij(mbeg+j)
              g = wk(j) - hh*f
              wk(j) = g
              do k=1,j
                 rowij(jk) = rowij(jk) - f*wk(k) - g*rowij(mbeg+k)
                 jk = jk + 1
              enddo
           enddo
        endif

        do k=1,l
           rowij(mbeg+k) = scale*rowij(mbeg+k)
        enddo
     end if
     root(i) = rowij(mbeg+i)
     rowij(mbeg+i) = h*scale*scale
     mbeg = mbeg - i + 1
     mm = mm - i
  enddo

!+---------------------------------------------------------------------+
!|          loop-210 compute eigenvalues and eigenvectors              |
!|          setup work area location eigv to the identity matrix       |
!|          loop-140 for finding small sub-diagonal element            |
!|          loop-160 for convergence of eigenvalue j (max. 30 times)   |
!|          loop-190 for ql transformation and loop-180 form vectors   |
!+---------------------------------------------------------------------+
  do i=1,m-1
     wk(i) = wk(i+1)
  enddo
  wk(m) = 0.d0; b = 0.d0; f = 0.d0
  eigv(1:m,1:m) = 0.d0
  do i=1,m
     eigv(i,i) = 1.d0
  enddo

! (loop-210)
  do l=1,m
     h = rdelp*(abs(root(l))+abs(wk(l)))
     if (b < h) b = h
! (loop-140)
     do n=l,m
        k = n
        if (abs(wk(k)) <= b) exit
     enddo
     n = k

     if (n/=l)  then
        do j=1,30
           if (j==31) then
              ier = 128 + l
              return
           end if

           l1 = l + 1
           g = root(l); p = (root(l1)-g)/(wk(l)+wk(l))
           r = abs(p)
           if (rdelp*abs(p).lt.1.0) r = sqrt(p*p+1.0)
           root(l) = wk(l)/(p+sign(r,p))
           h = g - root(l)
           root(l1:m) = root(l1:m) - h
           f = f + h

           p = root(n); c = 1.d0; s = 0.d0
           nn1 = n - 1; nn1pl = nn1 + l
           if (l <= nn1) then
              do ii=l,nn1
                 i = nn1pl - ii
                 g = c*wk(i)
                 h = c*p
                 if (abs(p) < abs(wk(i))) then
                    c = p/wk(i)
                    r = sqrt(c*c+1.d0)
                    wk(i+1) = s*wk(i)*r
                    s = 1.d0/r
                    c = c*s
                 else
                    c = wk(i)/p
                    r = sqrt(c*c+1.d0)
                    wk(i+1) = s*p*r
                    s = c/r
                    c = 1.d0/r
                 end if
                 p = c*root(i) - s*g
                 root(i+1) = h + s*(c*g+s*root(i))
                 if (ni >= m) then
                    do k=1,m
                       h = eigv(k,i+1)
                       eigv(k,i+1) = s*eigv(k,i) + c*h
                       eigv(k,i) = c*eigv(k,i) - s*h
                    enddo
                 end if
              enddo
           end if
           wk(l) = s*p; root(l) = c*p
           if (abs(wk(l)) <= b) exit
        enddo
     endif
     root(l) = root(l) + f
  enddo

!+---------------------------------------------------------------------+
!|          back transform eigenvectors of the original matrix from    |
!|          eigenvectors 1 to m of the symmetric tridiagonal matrix    |
!+---------------------------------------------------------------------+
  do i=2,m
     l = i - 1
     ia = (i*l)/2
     if (abs(rowij(ia+i)) > 0.0) then
        do j=1,m
           sum1=sum(rowij(ia+1:ia+l)*eigv(1:l,j))/rowij(ia+i)
           do k=1,l
              eigv(k,j) = eigv(k,j) - sum1*rowij(ia+k)
           enddo
        enddo
     end if
  enddo

  return
end subroutine symtrx
